<?php

declare(strict_types=1);

namespace Drupal\ai_content_creator\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Serialization\Json;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Service for handling OpenAI API interactions.
 */
class OpenAiService {

  use StringTranslationTrait;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new OpenAiService object.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * Generate content using OpenAI API.
   *
   * @param string $prompt
   *   The content prompt.
   *
   * @return array
   *   Array with 'success' boolean and either 'content' or 'error' message.
   */
  public function generateContent(string $prompt): array {
    $config = $this->configFactory->get('ai_content_creator.adminsettings');
    
    // Validate configuration.
    $api_key = $config->get('api_key');
    if (empty($api_key)) {
      return [
        'success' => FALSE,
        'error' => $this->t('OpenAI API key is not configured.'),
      ];
    }

    $model = $config->get('api_model') ?? 'gpt-3.5-turbo';
    $max_tokens = (int) $config->get('api_max_token') ?? 2000;
    $temperature = (float) $config->get('api_temperature') ?? 0.7;
    $url = $config->get('api_url') ?? 'https://api.openai.com/v1/completions';

    // Prepare request based on model type.
    $is_chat_model = strpos($model, 'gpt-3.5') !== FALSE || strpos($model, 'gpt-4') !== FALSE;
    
    if ($is_chat_model) {
      $url = str_replace('/completions', '/chat/completions', $url);
      if (!str_contains($url, 'chat/completions')) {
        $url = rtrim($url, '/') . '/chat/completions';
      }
      $payload = [
        'model' => $model,
        'messages' => [
          [
            'role' => 'user',
            'content' => $prompt,
          ],
        ],
        'temperature' => $temperature,
        'max_tokens' => $max_tokens,
      ];
    } else {
      $payload = [
        'model' => $model,
        'prompt' => $prompt,
        'temperature' => $temperature,
        'max_tokens' => $max_tokens,
      ];
    }

    $options = [
      'headers' => [
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $api_key,
      ],
      'json' => $payload,
      'timeout' => 30,
    ];

    try {
      $response = $this->httpClient->request('POST', $url, $options);
      $result = $response->getBody()->getContents();
      $data = Json::decode($result);

      // Check for API errors.
      if (isset($data['error'])) {
        $error_message = $data['error']['message'] ?? 'Unknown API error';
        $this->loggerFactory->get('ai_content_creator')->error('OpenAI API Error: @error', ['@error' => $error_message]);
        return [
          'success' => FALSE,
          'error' => $this->t('OpenAI API Error: @error', ['@error' => $error_message]),
        ];
      }

      // Validate response structure.
      if (!isset($data['choices']) || empty($data['choices'])) {
        return [
          'success' => FALSE,
          'error' => $this->t('Invalid response from OpenAI API.'),
        ];
      }

      // Extract content based on model type.
      if ($is_chat_model) {
        $content = $data['choices'][0]['message']['content'] ?? '';
      } else {
        $content = $data['choices'][0]['text'] ?? '';
      }

      $content = trim($content);
      if (empty($content)) {
        return [
          'success' => FALSE,
          'error' => $this->t('No content was generated. Please try a different prompt.'),
        ];
      }

      return [
        'success' => TRUE,
        'content' => $content,
      ];

    }
    catch (RequestException $exception) {
      $status_code = $exception->hasResponse() ? $exception->getResponse()->getStatusCode() : 'Unknown';
      $error_msg = $exception->getMessage();
      
      // Log the detailed error for debugging.
      $this->loggerFactory->get('ai_content_creator')->error('OpenAI API Request Exception: @error (Status: @status)', [
        '@error' => $error_msg,
        '@status' => $status_code,
      ]);

      // Provide user-friendly error messages.
      switch ($status_code) {
        case 401:
          $user_message = $this->t('Invalid API key. Please check your OpenAI API configuration.');
          break;
        case 429:
          $user_message = $this->t('Rate limit exceeded. Please try again later.');
          break;
        case 500:
        case 502:
        case 503:
          $user_message = $this->t('OpenAI service is temporarily unavailable. Please try again later.');
          break;
        default:
          $user_message = $this->t('Connection error occurred. Please try again.');
      }

      return [
        'success' => FALSE,
        'error' => $user_message,
      ];
    }
  }

  /**
   * Validate API configuration.
   *
   * @return array
   *   Array with 'valid' boolean and 'errors' array.
   */
  public function validateConfiguration(): array {
    $config = $this->configFactory->get('ai_content_creator.adminsettings');
    $errors = [];

    if (empty($config->get('api_key'))) {
      $errors[] = $this->t('API key is required.');
    }

    if (empty($config->get('api_url'))) {
      $errors[] = $this->t('API URL is required.');
    }

    $max_tokens = $config->get('api_max_token');
    if (empty($max_tokens) || !is_numeric($max_tokens) || $max_tokens < 1) {
      $errors[] = $this->t('Maximum tokens must be a positive number.');
    }

    return [
      'valid' => empty($errors),
      'errors' => $errors,
    ];
  }

} 